from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.energiemix import Energiemix
from ..com.externe_referenz import ExterneReferenz
from ..com.vertragskonditionen import Vertragskonditionen
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.kundentyp import Kundentyp
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.tarifmerkmal import Tarifmerkmal
from ..enum.tariftyp import Tariftyp
from .kosten import Kosten
from .marktteilnehmer import Marktteilnehmer


class Tarifkosten(BaseModel):
    """
    Objekt zur Kommunikation von Kosten, die im Rahmen der Tarifanwendung entstehen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarifkosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifkosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Tarifkosten.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    anbieter: Marktteilnehmer | None = None
    anbietername: Annotated[str | None, Field(None, title="Anbietername")]
    anwendung_von: Annotated[datetime | None, Field(None, alias="anwendungVon", title="Anwendungvon")]
    bemerkung: Annotated[str | None, Field(None, title="Bemerkung")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.TARIFKOSTEN, alias="boTyp")]
    energiemix: Energiemix | None = None
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    kosten: Kosten | None = None
    kundentypen: Annotated[list[Kundentyp] | None, Field(None, title="Kundentypen")]
    sparte: Sparte | None = None
    tarifart: Tarifart | None = None
    tarifmerkmale: Annotated[list[Tarifmerkmal] | None, Field(None, title="Tarifmerkmale")]
    tariftyp: Tariftyp | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vertragskonditionen: Vertragskonditionen | None = None
    website: Annotated[str | None, Field(None, title="Website")]
    zeitliche_gueltigkeit: Annotated[Zeitraum | None, Field(None, alias="zeitlicheGueltigkeit")]



from enum import Enum


class MesstechnischeEinordnung(str, Enum):
    """
    An enum for the messtechnische Einordnung
    """

    IMS = "IMS"
    KME_MME = "KME_MME"
    KEINE_MESSUNG = "KEINE_MESSUNG"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp


class Dokument(BaseModel):
    """
    A generic document reference like for bills, order confirmations and cancellations
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.GESCHAEFTSOBJEKT, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    erstellungsdatum: Annotated[datetime, Field(title="Erstellungsdatum")]
    has_been_sent: Annotated[bool, Field(alias="hasBeenSent", title="Hasbeensent")]
    dokumentenname: Annotated[str, Field(title="Dokumentenname")]
    vorlagenname: Annotated[str, Field(title="Vorlagenname")]



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.ablesende_rolle import AblesendeRolle
from ..enum.ablesungsstatus import Ablesungsstatus
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.messwertstatus import Messwertstatus
from ..enum.wertermittlungsverfahren import Wertermittlungsverfahren


class Verbrauch(BaseModel):
    """
    Abbildung eines zeitlich abgegrenzten Verbrauchs

    .. raw:: html

        <object data="../_static/images/bo4e/com/Verbrauch.svg" type="image/svg+xml"></object>

    .. HINT::
        `Verbrauch JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Verbrauch.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    einheit: Mengeneinheit | None = None
    enddatum: Annotated[datetime | None, Field(None, title="Enddatum")]
    obis_kennzahl: Annotated[str | None, Field(None, alias="obisKennzahl", title="Obiskennzahl")]
    startdatum: Annotated[datetime | None, Field(None, title="Startdatum")]
    wert: Annotated[Decimal | None, Field(None, title="Wert")]
    wertermittlungsverfahren: Wertermittlungsverfahren | None = None
    ablesegrund: Annotated[str | None, Field(None, title="Ablesegrund")]
    ablesebeschreibung: Annotated[str | None, Field(None, title="Ablesebeschreibung")]
    periodenverbrauch: Annotated[Decimal | None, Field(None, title="Periodenverbrauch")]
    periodenverbrauch_ursprung: Annotated[
        str | None, Field(None, alias="periodenverbrauchUrsprung", title="Periodenverbrauchursprung")
    ]
    ableser: AblesendeRolle | None = None
    status: Ablesungsstatus | None = None
    energiegehalt_gas: Annotated[Decimal | None, Field(None, alias="energiegehaltGas", title="Energiegehaltgas")]
    energiegehalt_gas_gueltig_von: Annotated[
        datetime | None, Field(None, alias="energiegehaltGasGueltigVon", title="Energiegehaltgasgueltigvon")
    ]
    energiegehalt_gas_gueltig_bis: Annotated[
        datetime | None, Field(None, alias="energiegehaltGasGueltigBis", title="Energiegehaltgasgueltigbis")
    ]
    umwandlungsfaktor_gas: Annotated[
        Decimal | None, Field(None, alias="umwandlungsfaktorGas", title="Umwandlungsfaktorgas")
    ]
    umwandlungsfaktor_gas_gueltig_von: Annotated[
        datetime | None, Field(None, alias="umwandlungsfaktorGasGueltigVon", title="Umwandlungsfaktorgasgueltigvon")
    ]
    umwandlungsfaktor_gas_gueltig_bis: Annotated[
        datetime | None, Field(None, alias="umwandlungsfaktorGasGueltigBis", title="Umwandlungsfaktorgasgueltigbis")
    ]
    messwertstatus: Messwertstatus | None = None



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.voraussetzungen import Voraussetzungen
from .geraet import Geraet
from .menge import Menge


class Tarifeinschraenkung(BaseModel):
    """
    Mit dieser Komponente werden Einschränkungen für die Anwendung von Tarifen modelliert.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifeinschraenkung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifeinschraenkung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Tarifeinschraenkung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    einschraenkungleistung: Annotated[list[Menge] | None, Field(None, title="Einschraenkungleistung")]
    einschraenkungzaehler: Annotated[list[Geraet] | None, Field(None, title="Einschraenkungzaehler")]
    voraussetzungen: Annotated[list[Voraussetzungen] | None, Field(None, title="Voraussetzungen")]
    zusatzprodukte: Annotated[list[str] | None, Field(None, title="Zusatzprodukte")]



from enum import Enum


class Geschaeftspartnerrolle(str, Enum):
    """
    Diese Rollen kann ein Geschäftspartner einnehmen.
    """

    LIEFERANT = "LIEFERANT"
    DIENSTLEISTER = "DIENSTLEISTER"
    KUNDE = "KUNDE"
    INTERESSENT = "INTERESSENT"
    MARKTPARTNER = "MARKTPARTNER"



from enum import Enum


class Wertermittlungsverfahren(str, Enum):
    """
    Gibt an, ob es sich um eine Prognose oder eine Messung handelt, beispielsweise bei der Abbildung eines Verbrauchs.
    """

    PROGNOSE = "PROGNOSE"
    MESSUNG = "MESSUNG"



"""
This module contains the base class
"""
import warnings
from inspect import currentframe, getframeinfo
from typing import Any, Optional, TypeVar

from bomf.model import Bo4eDataSet

Model = TypeVar("Model", bound=Bo4eDataSet)


class DataSetBaseModel(Bo4eDataSet):
    """
    This class serves as base class for all dataset classes in this module. The main purpose for now is to override
    the `construct` method such that it logs a warning if the construct method is used but proceeds normally.
    """

    @classmethod
    def model_construct(cls: type[Model], _fields_set: Optional[set[str]] = None, **values: Any) -> Model:
        """
        Method definition copied from pydantic `construct(...)` method.
        It only logs a warning if this method is used but proceeds normally.
        """
        current_frame = currentframe()
        assert current_frame is not None
        call_frame = current_frame.f_back
        assert call_frame is not None
        call_frame_info = getframeinfo(call_frame)

        # filepath = Path(call_frame_info.filename).relative_to(Path(__file__).parents[5])
        warning_msg = (
            "Avoid using construct on datasets."
            + f" Used in {call_frame_info.function}, {call_frame_info.filename}:{call_frame_info.lineno}"
        )
        warnings.warn(
            warning_msg,
            category=DeprecationWarning,
        )
        # raise DeprecationWarning(warning_msg)
        # Uncomment this raise statement if you want to crash your tests on unwanted construct statements in your code.
        return super(DataSetBaseModel, cls).model_construct.__func__(  # type:ignore[attr-defined]
            cls, _fields_set, **values
        )



"""
Contains the dataset for the product loader.
It also contains the validation logic for the product loader dataset.
"""
from datetime import datetime
from decimal import Decimal
from typing import Optional

from ibims.bo4e import (
    Adresse,
    Kampagne,
    Messlokation,
    Preisblatt,
    Preisgarantie,
    Verbrauch,
    Vertragskonditionen,
    VertragskontoCBA,
)
from ibims.datasets import DataSetBaseModel


class TripicaProductLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e data set that consists of
    a Preisblatt currently
    """

    preisblattBasis: list[Preisblatt]

    kampagne: Optional[list[Kampagne]] = None  #: models the tripica base characteristics signup(id)

    preisblattVerbrauch: list[Preisblatt]

    preisblattAbschlag: Preisblatt  #: used for tripica payment; filled from powercloud contract(tariffs)+product

    preisblattZaehlpunkt: Preisblatt  #: used for tripica delivery point

    preisblattSED: Preisblatt
    # SED is short for "Service Enercity Digital". It's meant to be a product for "deutscher energiemarkt" specific data
    # See the documentation of `def _create_sed_product` for details.

    preisblattRabatt: Preisblatt
    """
    used for the tripica discount product; filled from powercloud contract(tariffs(flexbonus))+product
    """

    prozentualer_rabatt: Optional[Decimal] = None
    """
    the percentage for tripica discount; the values are percentages from 0 to 100
    """
    absoluter_rabatt: Optional[Decimal] = None
    """
    the percentage for tripica discount; the values are percentages from 0 to 100
    """

    # I originally thought about it integrating the absolute/prozentual rabatt nicely into the preisblatt rabatt.
    # Then I remembered that the intermediate model is f***ed up anyway, so I'll just leave the fields on root level.
    # I'll just leave it here for now. https://github.com/Hochfrequenz/powercloud2lynqtech/issues/937

    verbrauch: Verbrauch

    lieferadresse: Adresse  #: used in the tripica delivery point characteristics; derived from powercloud contract

    messlokation: Messlokation

    cba: VertragskontoCBA

    vertragskonditionen: list[Vertragskonditionen]
    """
    original duration of the contract/product; used for tripica "sed" product agreement items:
     * "INITIAL_COMMITMENT"
     * "TERMINATION_NOTICE_TIME_END_OF_MONTH"

    read from powercloud contract tariffs
    """
    initiale_preisgarantie: list[Preisgarantie]
    """
    initial price guarantee; used for tripica "sed" product agreement item "INITIAL_PRICE_GUARANTEE".
    read from powercloud contract tariffs
    """
    erstellungsdatum: Optional[datetime] = None
    """erstellungsdatum des dem dataset zugrunde liegenden vertrags"""
    naechster_abschlagstermin: datetime  # tripica: sed/nextRenewalDate; powercloud: deposit/nextUseAt
    """Der Termin zu dem der nächste Abschlag abgebucht wird"""



from enum import Enum


class Abgabeart(str, Enum):
    """
    This AbgabeArt models the Konzessionsabgabentyp.
    It contains concessionfee types needed for concessionFee mapping.
    """

    KAS = "KAS"
    SA = "SA"
    SAS = "SAS"
    TA = "TA"
    TAS = "TAS"
    TK = "TK"
    TKS = "TKS"
    TS = "TS"
    TSS = "TSS"



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.mengeneinheit import Mengeneinheit
from ..enum.preisstatus import Preisstatus
from ..enum.preistyp import Preistyp
from ..enum.waehrungseinheit import Waehrungseinheit


class Tarifpreis(BaseModel):
    """
    Abbildung eines Tarifpreises mit Preistyp und Beschreibung abgeleitet von COM Preis.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifpreis.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreis JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Tarifpreis.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bezugswert: Mengeneinheit | None = None
    einheit: Waehrungseinheit | None = None
    preistyp: Preistyp | None = None
    status: Preisstatus | None = None
    wert: Annotated[Decimal | None, Field(None, title="Wert")]



from enum import Enum


class Rechnungsstatus(str, Enum):
    """
    Abbildung verschiedener Zustände, die im Rahmen der Rechnungsbearbeitung durchlaufen werden.
    """

    UNGEPRUEFT = "UNGEPRUEFT"
    GEPRUEFT_OK = "GEPRUEFT_OK"
    GEPRUEFT_FEHLERHAFT = "GEPRUEFT_FEHLERHAFT"
    GEBUCHT = "GEBUCHT"
    BEZAHLT = "BEZAHLT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.preisgarantietyp import Preisgarantietyp
from .zeitraum import Zeitraum


class Preisgarantie(BaseModel):
    """
    Definition für eine Preisgarantie mit der Möglichkeit verschiedener Ausprägungen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preisgarantie.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisgarantie JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Preisgarantie.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    preisgarantietyp: Preisgarantietyp | None = None
    zeitliche_gueltigkeit: Annotated[Zeitraum | None, Field(None, alias="zeitlicheGueltigkeit")]
    creation_date: Annotated[datetime | None, Field(None, alias="creationDate", title="Creationdate")]



from enum import Enum


class Kontaktart(str, Enum):
    """
    Art des Kontaktes zwischen Geschäftspartnern.
    """

    ANSCHREIBEN = "ANSCHREIBEN"
    TELEFONAT = "TELEFONAT"
    FAX = "FAX"
    E_MAIL = "E_MAIL"
    SMS = "SMS"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .geraeteeigenschaften import Geraeteeigenschaften


class Geraet(BaseModel):
    """
    Mit dieser Komponente werden alle Geräte modelliert, die keine Zähler sind.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Geraet.svg" type="image/svg+xml"></object>

    .. HINT::
        `Geraet JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Geraet.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    geraeteeigenschaften: Geraeteeigenschaften | None = None
    geraetenummer: Annotated[str | None, Field(None, title="Geraetenummer")]



from enum import Enum


class Preistyp(str, Enum):
    """
    Aufschlüsselung der Preistypen in Tarifen.
    """

    GRUNDPREIS = "GRUNDPREIS"
    ARBEITSPREIS_EINTARIF = "ARBEITSPREIS_EINTARIF"
    ARBEITSPREIS_HT = "ARBEITSPREIS_HT"
    ARBEITSPREIS_NT = "ARBEITSPREIS_NT"
    LEISTUNGSPREIS = "LEISTUNGSPREIS"
    MESSPREIS = "MESSPREIS"
    ENTGELT_ABLESUNG = "ENTGELT_ABLESUNG"
    ENTGELT_ABRECHNUNG = "ENTGELT_ABRECHNUNG"
    ENTGELT_MSB = "ENTGELT_MSB"
    PROVISION = "PROVISION"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.tagesvektor import Tagesvektor
from ..com.zeitintervall import Zeitintervall
from ..enum.bo_typ import BoTyp
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.sparte import Sparte


class LastgangKompakt(BaseModel):
    """
    Modell zur Abbildung eines kompakten Lastganges.
    In diesem Modell werden die Messwerte in Form von Tagesvektoren mit fester Anzahl von Werten übertragen.
    Daher ist dieses BO nur zur Übertragung von äquidistanten Messwertverläufen geeignet.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.LASTGANG_KOMPAKT, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    lokations_id: Annotated[str | None, Field(None, alias="lokationsId", title="Lokationsid")]
    lokationstyp: Annotated[str | None, Field(None, title="Lokationstyp")]
    messgroesse: Mengeneinheit | None = None
    obis_kennzahl: Annotated[str | None, Field(None, alias="obisKennzahl", title="Obiskennzahl")]
    sparte: Sparte | None = None
    tagesvektoren: Annotated[list[Tagesvektor] | None, Field(None, title="Tagesvektoren")]
    version: Annotated[str | None, Field(None, title="Version")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    zeitintervall: Zeitintervall | None = None



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.ausschreibungslos import Ausschreibungslos
from ..com.externe_referenz import ExterneReferenz
from ..com.zeitraum import Zeitraum
from ..enum.ausschreibungsportal import Ausschreibungsportal
from ..enum.ausschreibungsstatus import Ausschreibungsstatus
from ..enum.ausschreibungstyp import Ausschreibungstyp
from ..enum.bo_typ import BoTyp
from .geschaeftspartner import Geschaeftspartner


class Ausschreibung(BaseModel):
    """
    Das BO Ausschreibung dient zur detaillierten Darstellung von ausgeschriebenen Energiemengen in der Energiewirtschaft

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Ausschreibung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ausschreibung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Ausschreibung.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    abgabefrist: Zeitraum | None = None
    ausschreibender: Geschaeftspartner | None = None
    ausschreibungportal: Ausschreibungsportal | None = None
    ausschreibungsnummer: Annotated[str | None, Field(None, title="Ausschreibungsnummer")]
    ausschreibungsstatus: Ausschreibungsstatus | None = None
    ausschreibungstyp: Ausschreibungstyp | None = None
    bindefrist: Zeitraum | None = None
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.AUSSCHREIUNG, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    kostenpflichtig: Annotated[bool | None, Field(None, title="Kostenpflichtig")]
    lose: Annotated[list[Ausschreibungslos] | None, Field(None, title="Lose")]
    veroeffentlichungszeitpunkt: Annotated[datetime | None, Field(None, title="Veroeffentlichungszeitpunkt")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    webseite: Annotated[str | None, Field(None, title="Webseite")]



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .sepa_info import SepaInfo


class Bankverbindung(BaseModel):
    """
    This component contains bank connection information.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    iban: Annotated[str | None, Field(None, title="Iban")]
    bic: Annotated[str | None, Field(None, title="Bic")]
    gueltig_seit: Annotated[datetime | None, Field(None, alias="gueltigSeit", title="Gueltigseit")]
    gueltig_bis: Annotated[datetime | None, Field(None, alias="gueltigBis", title="Gueltigbis")]
    bankname: Annotated[str | None, Field(None, title="Bankname")]
    sepa_info: Annotated[SepaInfo | None, Field(None, alias="sepaInfo")]
    kontoinhaber: Annotated[str | None, Field(None, title="Kontoinhaber")]
    ouid: Annotated[int, Field(title="Ouid")]



from enum import Enum


class Gebiettyp(str, Enum):
    """
    List of possible Gebiettypen.
    """

    REGELZONE = "REGELZONE"
    MARKTGEBIET = "MARKTGEBIET"
    BILANZIERUNGSGEBIET = "BILANZIERUNGSGEBIET"
    VERTEILNETZ = "VERTEILNETZ"
    TRANSPORTNETZ = "TRANSPORTNETZ"
    REGIONALNETZ = "REGIONALNETZ"
    AREALNETZ = "AREALNETZ"
    GRUNDVERSORGUNGSGEBIET = "GRUNDVERSORGUNGSGEBIET"
    VERSORGUNGSGEBIET = "VERSORGUNGSGEBIET"



from enum import Enum


class Ausschreibungsstatus(str, Enum):
    """
    Bezeichnungen für die Ausschreibungsphasen
    """

    PHASE1 = "PHASE1"
    PHASE2 = "PHASE2"
    PHASE3 = "PHASE3"
    PHASE4 = "PHASE4"



from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.abgabeart import Abgabeart
from ..enum.energierichtung import Energierichtung
from ..enum.mengeneinheit import Mengeneinheit


class Zaehlwerk(BaseModel):
    """
    Mit dieser Komponente werden Zählwerke modelliert.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zaehlwerk.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehlwerk JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Zaehlwerk.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    einheit: Mengeneinheit | None = None
    obis_kennzahl: Annotated[str | None, Field(None, alias="obisKennzahl", title="Obiskennzahl")]
    richtung: Energierichtung | None = None
    wandlerfaktor: Annotated[Decimal | None, Field(None, title="Wandlerfaktor")]
    zaehlwerk_id: Annotated[str | None, Field(None, alias="zaehlwerkId", title="Zaehlwerkid")]
    vorkommastellen: Annotated[int, Field(title="Vorkommastellen")]
    nachkommastellen: Annotated[int, Field(title="Nachkommastellen")]
    schwachlastfaehig: Annotated[bool, Field(title="Schwachlastfaehig")]
    konzessionsabgaben_typ: Annotated[Abgabeart | None, Field(None, alias="konzessionsabgabenTyp")]
    active_from: Annotated[datetime, Field(alias="activeFrom", title="Activefrom")]
    active_until: Annotated[datetime | None, Field(None, alias="activeUntil", title="Activeuntil")]
    description: Annotated[str | None, Field(None, title="Description")]
    verbrauchsart: Annotated[str | None, Field(None, title="Verbrauchsart")]



from enum import Enum


class AufAbschlagsziel(str, Enum):
    """
    Der Preis, auf den sich ein Auf- oder Abschlag bezieht.
    """

    ARBEITSPREIS_EINTARIF = "ARBEITSPREIS_EINTARIF"
    ARBEITSPREIS_HT = "ARBEITSPREIS_HT"
    ARBEITSPREIS_NT = "ARBEITSPREIS_NT"
    ARBEITSPREIS_HT_NT = "ARBEITSPREIS_HT_NT"
    GRUNDPREIS = "GRUNDPREIS"
    GESAMTPREIS = "GESAMTPREIS"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.angebotsvariante import Angebotsvariante
from ..com.externe_referenz import ExterneReferenz
from ..enum.bo_typ import BoTyp
from ..enum.sparte import Sparte
from .ansprechpartner import Ansprechpartner
from .geschaeftspartner import Geschaeftspartner


class Angebot(BaseModel):
    """
    Mit diesem BO kann ein Versorgungsangebot zur Strom- oder Gasversorgung oder die Teilnahme an einer Ausschreibung
    übertragen werden. Es können verschiedene Varianten enthalten sein (z.B. ein- und mehrjährige Laufzeit).
    Innerhalb jeder Variante können Teile enthalten sein, die jeweils für eine oder mehrere Marktlokationen erstellt
    werden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Angebot.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebot JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Angebot.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    anfragereferenz: Annotated[str | None, Field(None, title="Anfragereferenz")]
    angebotsdatum: Annotated[datetime | None, Field(None, title="Angebotsdatum")]
    angebotsgeber: Geschaeftspartner | None = None
    angebotsnehmer: Geschaeftspartner | None = None
    angebotsnummer: Annotated[str | None, Field(None, title="Angebotsnummer")]
    bindefrist: Annotated[datetime | None, Field(None, title="Bindefrist")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ANGEBOT, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    sparte: Sparte | None = None
    unterzeichner_angebotsgeber: Annotated[Ansprechpartner | None, Field(None, alias="unterzeichnerAngebotsgeber")]
    unterzeichner_angebotsnehmer: Annotated[Ansprechpartner | None, Field(None, alias="unterzeichnerAngebotsnehmer")]
    varianten: Annotated[list[Angebotsvariante] | None, Field(None, title="Varianten")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from enum import Enum


class Bilanzierungsmethode(str, Enum):
    """
    Mit dieser Aufzählung kann zwischen den Bilanzierungsmethoden bzw. -grundlagen unterschieden werden.
    """

    RLM = "RLM"
    SLP = "SLP"
    TLP_GEMEINSAM = "TLP_GEMEINSAM"
    TLP_GETRENNT = "TLP_GETRENNT"
    PAUSCHAL = "PAUSCHAL"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.verbrauch import Verbrauch
from ..enum.bo_typ import BoTyp
from ..enum.lokationstyp import Lokationstyp


class Energiemenge(BaseModel):
    """
    Abbildung von Mengen, die Lokationen zugeordnet sind

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Energiemenge.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemenge JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Energiemenge.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.ENERGIEMENGE, alias="boTyp")]
    energieverbrauch: Annotated[list[Verbrauch] | None, Field(None, title="Energieverbrauch")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    lokations_id: Annotated[str | None, Field(None, alias="lokationsId", title="Lokationsid")]
    lokationstyp: Lokationstyp | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from enum import Enum


class Rufnummernart(str, Enum):
    """
    Abbildung verschiedener Rufnummerntypen.
    """

    RUF_ZENTRALE = "RUF_ZENTRALE"
    FAX_ZENTRALE = "FAX_ZENTRALE"
    SAMMELRUF = "SAMMELRUF"
    SAMMELFAX = "SAMMELFAX"
    ABTEILUNGRUF = "ABTEILUNGRUF"
    ABTEILUNGFAX = "ABTEILUNGFAX"
    RUF_DURCHWAHL = "RUF_DURCHWAHL"
    FAX_DURCHWAHL = "FAX_DURCHWAHL"
    MOBIL_NUMMER = "MOBIL_NUMMER"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .betrag import Betrag
from .fremdkostenposition import Fremdkostenposition


class Fremdkostenblock(BaseModel):
    """
    Komponente zur Abbildung eines Kostenblocks in den Fremdkosten

    .. raw:: html

        <object data="../_static/images/bo4e/com/Fremdkostenblock.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkostenblock JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Fremdkostenblock.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    kostenblockbezeichnung: Annotated[str | None, Field(None, title="Kostenblockbezeichnung")]
    kostenpositionen: Annotated[list[Fremdkostenposition] | None, Field(None, title="Kostenpositionen")]
    summe_kostenblock: Annotated[Betrag | None, Field(None, alias="summeKostenblock")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .auf_abschlagstaffel_pro_ort import AufAbschlagstaffelProOrt


class AufAbschlagProOrt(BaseModel):
    """
    Mit dieser Komponente können Auf- und Abschläge verschiedener Typen im Zusammenhang
    mit örtlichen Gültigkeiten abgebildet werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/AufAbschlagProOrt.svg" type="image/svg+xml"></object>

    .. HINT::
        `AufAbschlagProOrt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/AufAbschlagProOrt.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    netznr: Annotated[str | None, Field(None, title="Netznr")]
    ort: Annotated[str | None, Field(None, title="Ort")]
    postleitzahl: Annotated[str | None, Field(None, title="Postleitzahl")]
    staffeln: Annotated[list[AufAbschlagstaffelProOrt] | None, Field(None, title="Staffeln")]



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .betrag import Betrag
from .menge import Menge
from .preis import Preis


class Kostenposition(BaseModel):
    """
    Diese Komponente wird zur Übertagung der Details zu einer Kostenposition verwendet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Kostenposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kostenposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Kostenposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    artikelbezeichnung: Annotated[str | None, Field(None, title="Artikelbezeichnung")]
    artikeldetail: Annotated[str | None, Field(None, title="Artikeldetail")]
    betrag_kostenposition: Annotated[Betrag | None, Field(None, alias="betragKostenposition")]
    bis: Annotated[datetime | None, Field(None, title="Bis")]
    einzelpreis: Preis | None = None
    menge: Menge | None = None
    positionstitel: Annotated[str | None, Field(None, title="Positionstitel")]
    von: Annotated[datetime | None, Field(None, title="Von")]
    zeitmenge: Menge | None = None



from enum import Enum


class Preisgarantietyp(str, Enum):
    """
    Aufzählung der Möglichkeiten für die Vergabe von Preisgarantien
    """

    ALLE_PREISBESTANDTEILE_BRUTTO = "ALLE_PREISBESTANDTEILE_BRUTTO"
    ALLE_PREISBESTANDTEILE_NETTO = "ALLE_PREISBESTANDTEILE_NETTO"
    PREISBESTANDTEILE_OHNE_ABGABEN = "PREISBESTANDTEILE_OHNE_ABGABEN"
    NUR_ENERGIEPREIS = "NUR_ENERGIEPREIS"



from enum import Enum


class Messwertstatus(str, Enum):
    """
    The enum Messwertstatus contains the status of a meter reading. The go implementation of this enum
    can be found in
    https://github.com/Hochfrequenz/go-bo4e/blob/86a2948b99be591db84039ebaf61ca3dff9bb1b0/enum/messwertstatus/messwertstatus.go
    """

    ABGELESEN = "ABGELESEN"
    ERSATZWERT = "ERSATZWERT"
    VORLAEUFIGERWERT = "VORLAEUFIGERWERT"
    ANGABE_FUER_LIEFERSCHEIN = "ANGABE_FUER_LIEFERSCHEIN"
    VORSCHLAGSWERT = "VORSCHLAGSWERT"
    NICHT_VERWENDBAR = "NICHT_VERWENDBAR"
    PROGNOSEWERT = "PROGNOSEWERT"
    ENERGIEMENGESUMMIERT = "ENERGIEMENGESUMMIERT"
    FEHLT = "FEHLT"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.angebotsstatus import Angebotsstatus
from .angebotsteil import Angebotsteil
from .betrag import Betrag
from .menge import Menge


class Angebotsvariante(BaseModel):
    """
    Führt die verschiedenen Ausprägungen der Angebotsberechnung auf

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsvariante.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsvariante JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Angebotsvariante.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    angebotsstatus: Angebotsstatus | None = None
    bindefrist: Annotated[datetime | None, Field(None, title="Bindefrist")]
    erstellungsdatum: Annotated[datetime | None, Field(None, title="Erstellungsdatum")]
    gesamtkosten: Betrag | None = None
    gesamtmenge: Menge | None = None
    teile: Annotated[list[Angebotsteil] | None, Field(None, title="Teile")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.zaehlertyp import Zaehlertyp
from .adresse import Adresse
from .menge import Menge
from .zeitraum import Zeitraum


class Ausschreibungsdetail(BaseModel):
    """
    Die Komponente Ausschreibungsdetail wird verwendet um die Informationen zu einer Abnahmestelle innerhalb eines
    Ausschreibungsloses abzubilden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Ausschreibungsdetail.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ausschreibungsdetail JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Ausschreibungsdetail.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    kunde: Annotated[str | None, Field(None, title="Kunde")]
    lastgang_vorhanden: Annotated[bool | None, Field(None, alias="lastgangVorhanden", title="Lastgangvorhanden")]
    lieferzeitraum: Zeitraum | None = None
    marktlokations_id: Annotated[str | None, Field(None, alias="marktlokationsId", title="Marktlokationsid")]
    marktlokationsadresse: Adresse | None = None
    marktlokationsbezeichnung: Annotated[str | None, Field(None, title="Marktlokationsbezeichnung")]
    netzbetreiber: Annotated[str | None, Field(None, title="Netzbetreiber")]
    netzebene_lieferung: Annotated[str | None, Field(None, alias="netzebeneLieferung", title="Netzebenelieferung")]
    netzebene_messung: Annotated[str | None, Field(None, alias="netzebeneMessung", title="Netzebenemessung")]
    prognose_arbeit_lieferzeitraum: Annotated[Menge | None, Field(None, alias="prognoseArbeitLieferzeitraum")]
    prognose_jahresarbeit: Annotated[Menge | None, Field(None, alias="prognoseJahresarbeit")]
    prognose_leistung: Annotated[Menge | None, Field(None, alias="prognoseLeistung")]
    rechnungsadresse: Adresse | None = None
    zaehlernummer: Annotated[str | None, Field(None, title="Zaehlernummer")]
    zaehlertechnik: Zaehlertyp | None = None



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.oekolabel import Oekolabel
from ..enum.oekozertifikat import Oekozertifikat
from ..enum.sparte import Sparte
from .energieherkunft import Energieherkunft


class Energiemix(BaseModel):
    """
    Zusammensetzung der gelieferten Energie aus den verschiedenen Primärenergieformen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Energiemix.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemix JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Energiemix.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    anteil: Annotated[list[Energieherkunft] | None, Field(None, title="Anteil")]
    atommuell: Annotated[Decimal | None, Field(None, title="Atommuell")]
    bemerkung: Annotated[str | None, Field(None, title="Bemerkung")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    co2_emission: Annotated[Decimal | None, Field(None, alias="co2Emission", title="Co2Emission")]
    energieart: Sparte | None = None
    energiemixnummer: Annotated[int | None, Field(None, title="Energiemixnummer")]
    gueltigkeitsjahr: Annotated[int | None, Field(None, title="Gueltigkeitsjahr")]
    oeko_top_ten: Annotated[bool | None, Field(None, alias="oekoTopTen", title="Oekotopten")]
    oekolabel: Annotated[list[Oekolabel] | None, Field(None, title="Oekolabel")]
    oekozertifikate: Annotated[list[Oekozertifikat] | None, Field(None, title="Oekozertifikate")]
    website: Annotated[str | None, Field(None, title="Website")]



from enum import Enum


class Messgroesse(str, Enum):
    """
    Gibt die physikalische Größe an, die gemessen wurde.
    """

    STROM = "STROM"
    SPANNUNG = "SPANNUNG"
    WIRKLEISTUNG = "WIRKLEISTUNG"
    BLINDLEISTUNG = "BLINDLEISTUNG"
    DRUCK = "DRUCK"
    LASTGANG = "LASTGANG"
    LASTPROFIL = "LASTPROFIL"
    TEMPERATUR = "TEMPERATUR"
    ZZAHL = "ZZAHL"
    BRENNWERT = "BRENNWERT"
    GRADTZAGSZAHLEN = "GRADTZAGSZAHLEN"
    VOLUMENSTROM = "VOLUMENSTROM"
    PREISE = "PREISE"



from enum import Enum


class NNRechnungstyp(str, Enum):
    """
    Abbildung verschiedener in der INVOIC angegebenen Rechnungstypen.
    """

    ABSCHLUSSRECHNUNG = "ABSCHLUSSRECHNUNG"
    ABSCHLAGSRECHNUNG = "ABSCHLAGSRECHNUNG"
    TURNUSRECHNUNG = "TURNUSRECHNUNG"
    MONATSRECHNUNG = "MONATSRECHNUNG"
    WIMRECHNUNG = "WIMRECHNUNG"
    ZWISCHENRECHNUNG = "ZWISCHENRECHNUNG"
    INTEGRIERTE_13_TE_RECHNUNG = "INTEGRIERTE_13TE_RECHNUNG"
    ZUSAETZLICHE_13_TE_RECHNUNG = "ZUSAETZLICHE_13TE_RECHNUNG"
    MEHRMINDERMENGENRECHNUNG = "MEHRMINDERMENGENRECHNUNG"



from enum import Enum


class Gueltigkeitstyp(str, Enum):
    """
    Übersicht der verschiedenen Gültigkeiten zur Umsetzung von Positiv- bzw. Negativlisten.
    """

    NUR_IN = "NUR_IN"
    NICHT_IN = "NICHT_IN"
    NUR_IN_KOMBINATION_MIT = "NUR_IN_KOMBINATION_MIT"



"""
This script checks if the current BO4E version is up-to-date.
"""
import sys

import click
import requests
from dotenv import dotenv_values, set_key

OWNER = "Hochfrequenz"
REPO = "BO4E-Schemas"
TIMEOUT = 10  # in seconds
DOTENV_FILE = "bo4e/tox.env"


def resolve_latest_version() -> str:
    """
    Resolve the latest BO4E version from the github api.
    Returns the version tag as string.
    E.g. "v2.0.13-rc3"
    """
    response = requests.get(f"https://api.github.com/repos/{OWNER}/{REPO}/releases/latest", timeout=TIMEOUT)
    response.raise_for_status()
    return response.json()["tag_name"]


def current_version() -> str:
    """
    Query the current version from the tox.env file.
    The version tag should have the same format as the github release tag.
    E.g. "v2.0.13-rc3"
    """
    config = dotenv_values(DOTENV_FILE)
    assert config["BO4E_VERSION"] is not None
    return config["BO4E_VERSION"]


@click.command()
def main():
    """
    Check if the current version is up-to-date. If so, exit with exit code 0.
    If not, update the version in the tox.env file and exit with exit code 1.
    """
    latest_version = resolve_latest_version()
    current = current_version()
    if latest_version == current:
        print(f"Version {current} is up to date.")
        return

    # Update BO4E:
    set_key(DOTENV_FILE, "BO4E_VERSION", latest_version)
    sys.exit(1)


if __name__ == "__main__":
    main()



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.adresse import Adresse
from ..com.externe_referenz import ExterneReferenz
from ..com.geokoordinaten import Geokoordinaten
from ..com.katasteradresse import Katasteradresse
from ..com.messlokationszuordnung import Messlokationszuordnung
from ..enum.bilanzierungsmethode import Bilanzierungsmethode
from ..enum.bo_typ import BoTyp
from ..enum.energierichtung import Energierichtung
from ..enum.gasqualitaet import Gasqualitaet
from ..enum.gebiettyp import Gebiettyp
from ..enum.kundentyp import Kundentyp
from ..enum.marktgebiet import Marktgebiet
from ..enum.messtechnische_einordnung import MesstechnischeEinordnung
from ..enum.netzebene import Netzebene
from ..enum.profiltyp import Profiltyp
from ..enum.prognosegrundlage import Prognosegrundlage
from ..enum.regelzone import Regelzone
from ..enum.sparte import Sparte
from ..enum.variant import Variant
from ..enum.verbrauchsart import Verbrauchsart
from .geschaeftspartner import Geschaeftspartner


class Marktlokation(BaseModel):
    """
    Object containing information about a Marktlokation

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Marktlokation.svg" type="image/svg+xml"></object>

    .. HINT::
        `Marktlokation JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Marktlokation.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bilanzierungsgebiet: Annotated[str | None, Field(None, title="Bilanzierungsgebiet")]
    bilanzierungsmethode: Bilanzierungsmethode | None = None
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.MARKTLOKATION, alias="boTyp")]
    endkunde: Geschaeftspartner | None = None
    energierichtung: Energierichtung | None = None
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    gasqualitaet: Gasqualitaet | None = None
    gebietstyp: Gebiettyp | None = None
    geoadresse: Geokoordinaten | None = None
    grundversorgercodenr: Annotated[str | None, Field(None, title="Grundversorgercodenr")]
    katasterinformation: Katasteradresse | None = None
    kundengruppen: Annotated[list[Kundentyp] | None, Field(None, title="Kundengruppen")]
    lokationsadresse: Adresse | None = None
    marktlokations_id: Annotated[str | None, Field(None, alias="marktlokationsId", title="Marktlokationsid")]
    netzbetreibercodenr: Annotated[str | None, Field(None, title="Netzbetreibercodenr")]
    netzebene: Netzebene | None = None
    netzgebietsnr: Annotated[str | None, Field(None, title="Netzgebietsnr")]
    sparte: Sparte | None = None
    unterbrechbar: Annotated[bool | None, Field(None, title="Unterbrechbar")]
    verbrauchsart: Verbrauchsart | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    zugehoerige_messlokation: Annotated[Messlokationszuordnung | None, Field(None, alias="zugehoerigeMesslokation")]
    messtechnische_einordnung: Annotated[MesstechnischeEinordnung, Field(alias="messtechnischeEinordnung")]
    uebertragungsnetzgebiet: Regelzone | None = None
    marktgebiet: Marktgebiet | None = None
    variant: Variant
    community_id: Annotated[str, Field(alias="communityId", title="Communityid")]
    prognose_grundlage: Annotated[Prognosegrundlage | None, Field(None, alias="prognoseGrundlage")]
    prognose_grundlage_detail: Annotated[Profiltyp | None, Field(None, alias="prognoseGrundlageDetail")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.zeitreihenwert import Zeitreihenwert
from ..enum.bo_typ import BoTyp
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.sparte import Sparte


class Lastgang(BaseModel):
    """
    Modell zur Abbildung eines Lastganges;
    In diesem Modell werden die Messwerte mit einem vollständigen Zeitintervall angegeben und es bietet daher eine hohe
    Flexibilität in der Übertragung jeglicher zeitlich veränderlicher Messgrössen.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Lastgang.svg" type="image/svg+xml"></object>

    .. HINT::
        `Lastgang JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Lastgang.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.LASTGANG, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    lokations_id: Annotated[str | None, Field(None, alias="lokationsId", title="Lokationsid")]
    lokationstyp: Annotated[str | None, Field(None, title="Lokationstyp")]
    messgroesse: Mengeneinheit | None = None
    obis_kennzahl: Annotated[str | None, Field(None, alias="obisKennzahl", title="Obiskennzahl")]
    sparte: Sparte | None = None
    version: Annotated[str | None, Field(None, title="Version")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    werte: Annotated[list[Zeitreihenwert] | None, Field(None, title="Werte")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.betrag import Betrag
from ..com.externe_referenz import ExterneReferenz
from ..com.fremdkostenblock import Fremdkostenblock
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp


class Fremdkosten(BaseModel):
    """
    Mit diesem BO werden die Fremdkosten, beispielsweise für eine Angebotserstellung oder eine Rechnungsprüfung,
    übertragen.
    Die Fremdkosten enthalten dabei alle Kostenblöcke, die von anderen Marktteilnehmern oder Instanzen erhoben werden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Fremdkosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Fremdkosten.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.FREMDKOSTEN, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    gueltigkeit: Zeitraum | None = None
    kostenbloecke: Annotated[list[Fremdkostenblock] | None, Field(None, title="Kostenbloecke")]
    summe_kosten: Annotated[Betrag | None, Field(None, alias="summeKosten")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



"""
Contains the dataset for the agent hint loader.
"""

from ibims.bo4e import Geschaeftspartner, Hinweis, Vertrag
from ibims.datasets.base import DataSetBaseModel


class ED4AgentHintLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e data set that consists of
    * a GeschaeftspartnerErweitert
    * a Vertrag
    * a hint
    In the context of this package is may be used to create ED$ AgentHint Data.
    """

    geschaeftspartner: Geschaeftspartner
    """
    The following attribute needs to be filled for this DataSet:
    - externe_referenzen
    - customerID
    """

    vertrag: Vertrag
    """
    Each Vertrag needs the following Attribute:
    - vertragsnummer
    """

    hinweis: Hinweis
    """
    A hint to specialties in the contract
    """



from enum import Enum


class Dienstleistungstyp(str, Enum):
    """
    Auflistung möglicher abzurechnender Dienstleistungen.
    """

    DATENBEREITSTELLUNG_TAEGLICH = "DATENBEREITSTELLUNG_TAEGLICH"
    DATENBEREITSTELLUNG_WOECHENTLICH = "DATENBEREITSTELLUNG_WOECHENTLICH"
    DATENBEREITSTELLUNG_MONATLICH = "DATENBEREITSTELLUNG_MONATLICH"
    DATENBEREITSTELLUNG_JAEHRLICH = "DATENBEREITSTELLUNG_JAEHRLICH"
    DATENBEREITSTELLUNG_HISTORISCHE_LG = "DATENBEREITSTELLUNG_HISTORISCHE_LG"
    DATENBEREITSTELLUNG_STUENDLICH = "DATENBEREITSTELLUNG_STUENDLICH"
    DATENBEREITSTELLUNG_VIERTELJAEHRLICH = "DATENBEREITSTELLUNG_VIERTELJAEHRLICH"
    DATENBEREITSTELLUNG_HALBJAEHRLICH = "DATENBEREITSTELLUNG_HALBJAEHRLICH"
    DATENBEREITSTELLUNG_MONATLICH_ZUSAETZLICH = "DATENBEREITSTELLUNG_MONATLICH_ZUSAETZLICH"
    DATENBEREITSTELLUNG_EINMALIG = "DATENBEREITSTELLUNG_EINMALIG"
    AUSLESUNG_2_X_TAEGLICH_FERNAUSLESUNG = "AUSLESUNG_2X_TAEGLICH_FERNAUSLESUNG"
    AUSLESUNG_TAEGLICH_FERNAUSLESUNG = "AUSLESUNG_TAEGLICH_FERNAUSLESUNG"
    AUSLESUNG_MANUELL_MSB = "AUSLESUNG_MANUELL_MSB"
    AUSLESUNG_MONATLICH_FERNAUSLESUNG = "AUSLESUNG_MONATLICH_FERNAUSLESUNG"
    AUSLESUNG_JAEHRLICH_FERNAUSLESUNG = "AUSLESUNG_JAEHRLICH_FERNAUSLESUNG"
    AUSLESUNG_MDE = "AUSLESUNG_MDE"
    ABLESUNG_MONATLICH = "ABLESUNG_MONATLICH"
    ABLESUNG_VIERTELJAEHRLICH = "ABLESUNG_VIERTELJAEHRLICH"
    ABLESUNG_HALBJAEHRLICH = "ABLESUNG_HALBJAEHRLICH"
    ABLESUNG_JAEHRLICH = "ABLESUNG_JAEHRLICH"
    AUSLESUNG_FERNAUSLESUNG = "AUSLESUNG_FERNAUSLESUNG"
    ABLESUNG_ZUSAETZLICH_MSB = "ABLESUNG_ZUSAETZLICH_MSB"
    ABLESUNG_ZUSAETZLICH_KUNDE = "ABLESUNG_ZUSAETZLICH_KUNDE"
    AUSLESUNG_FERNAUSLESUNG_ZUSAETZLICH_MSB = "AUSLESUNG_FERNAUSLESUNG_ZUSAETZLICH_MSB"
    AUSLESUNG_MOATLICH_FERNAUSLESUNG = "AUSLESUNG_MOATLICH_FERNAUSLESUNG"
    AUSLESUNG_STUENDLICH_FERNAUSLESUNG = "AUSLESUNG_STUENDLICH_FERNAUSLESUNG"
    AUSLESUNG_TEMPERATURMENGENUMWERTER = "AUSLESUNG_TEMPERATURMENGENUMWERTER"
    AUSLESUNG_ZUSTANDSMENGENUMWERTER = "AUSLESUNG_ZUSTANDSMENGENUMWERTER"
    AUSLESUNG_SYSTEMMENGENUMWERTER = "AUSLESUNG_SYSTEMMENGENUMWERTER"
    AUSLESUNG_VORGANG = "AUSLESUNG_VORGANG"
    AUSLESUNG_KOMPAKTMENGENUMWERTER = "AUSLESUNG_KOMPAKTMENGENUMWERTER"
    SPERRUNG = "SPERRUNG"
    ENTSPERRUNG = "ENTSPERRUNG"
    MAHNKOSTEN = "MAHNKOSTEN"
    INKASSOKOSTEN = "INKASSOKOSTEN"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.auf_abschlag_regional import AufAbschlagRegional
from ..com.energiemix import Energiemix
from ..com.externe_referenz import ExterneReferenz
from ..com.preisgarantie import Preisgarantie
from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
from ..com.tarifeinschraenkung import Tarifeinschraenkung
from ..com.tarifpreisposition_pro_ort import TarifpreispositionProOrt
from ..com.vertragskonditionen import Vertragskonditionen
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.kundentyp import Kundentyp
from ..enum.sparte import Sparte
from ..enum.tarifart import Tarifart
from ..enum.tarifmerkmal import Tarifmerkmal
from ..enum.tariftyp import Tariftyp
from .marktteilnehmer import Marktteilnehmer


class Tarif(BaseModel):
    """
    Abbildung eines Tarifs mit regionaler Zuordnung von Preisen und Auf- und Abschlägen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarif.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarif JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Tarif.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    anbieter: Marktteilnehmer | None = None
    anbietername: Annotated[str | None, Field(None, title="Anbietername")]
    anwendung_von: Annotated[datetime | None, Field(None, alias="anwendungVon", title="Anwendungvon")]
    bemerkung: Annotated[str | None, Field(None, title="Bemerkung")]
    berechnungsparameter: Tarifberechnungsparameter | None = None
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.TARIF, alias="boTyp")]
    energiemix: Energiemix | None = None
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    kundentypen: Annotated[list[Kundentyp] | None, Field(None, title="Kundentypen")]
    preisgarantie: Preisgarantie | None = None
    preisstand: Annotated[datetime | None, Field(None, title="Preisstand")]
    sparte: Sparte | None = None
    tarif_auf_abschlaege: Annotated[
        list[AufAbschlagRegional] | None, Field(None, alias="tarifAufAbschlaege", title="Tarifaufabschlaege")
    ]
    tarifart: Tarifart | None = None
    tarifeinschraenkung: Tarifeinschraenkung | None = None
    tarifmerkmale: Annotated[list[Tarifmerkmal] | None, Field(None, title="Tarifmerkmale")]
    tarifpreise: Annotated[list[TarifpreispositionProOrt] | None, Field(None, title="Tarifpreise")]
    tariftyp: Tariftyp | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vertragskonditionen: Vertragskonditionen | None = None
    website: Annotated[str | None, Field(None, title="Website")]
    zeitliche_gueltigkeit: Annotated[Zeitraum | None, Field(None, alias="zeitlicheGueltigkeit")]



from enum import Enum


class Tarifmerkmal(str, Enum):
    """
    Produktmerkmale im Zusammenhang mit der Tarifdefinition.
    """

    STANDARD = "STANDARD"
    VORKASSE = "VORKASSE"
    PAKET = "PAKET"
    KOMBI = "KOMBI"
    FESTPREIS = "FESTPREIS"
    BAUSTROM = "BAUSTROM"
    HAUSLICHT = "HAUSLICHT"
    HEIZSTROM = "HEIZSTROM"
    ONLINE = "ONLINE"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.rufnummernart import Rufnummernart


class Rufnummer(BaseModel):
    """
    Contains information to call or fax someone

    .. raw:: html

        <object data="../_static/images/bo4e/com/Rufnummer.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rufnummer JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Rufnummer.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    nummerntyp: Rufnummernart | None = None
    rufnummer: Annotated[str | None, Field(None, title="Rufnummer")]



"""
Bundles all datasets for the loaders
"""
from ibims.datasets.agent_hint_loader import ED4AgentHintLoaderDataSet
from ibims.datasets.base import DataSetBaseModel
from ibims.datasets.customer_loader import TripicaCustomerLoaderDataSet
from ibims.datasets.document_loader import TripicaDocumentLoaderDataSet
from ibims.datasets.invoice_loader import InvoiceLoaderDataSet
from ibims.datasets.network_loader import TripicaNetworkLoaderDataSet
from ibims.datasets.quantities_loader import QuantitiesLoaderDataSet
from ibims.datasets.resource_loader import TripicaResourceLoaderDataSet
from ibims.datasets.usage_loader import TripicaUsageLoaderDataSet



from enum import Enum


class Regelzone(str, Enum):
    """
    a strenum where the str-value of each member is the ILN of the Regelzone for electricity
    """

    FIELD_10_YDE_ENBW_____N = "10YDE-ENBW-----N"
    FIELD_10_YDE_EON______1 = "10YDE-EON------1"
    FIELD_10_YDE_RWENET___I = "10YDE-RWENET---I"
    FIELD_10_YDE_VE_______2 = "10YDE-VE-------2"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.preisposition import Preisposition
from ..com.zeitraum import Zeitraum
from ..enum.bo_typ import BoTyp
from ..enum.preisstatus import Preisstatus
from ..enum.sparte import Sparte
from .marktteilnehmer import Marktteilnehmer


class Preisblatt(BaseModel):
    """
    Das allgemeine Modell zur Abbildung von Preisen;
    Davon abgeleitet können, über die Zuordnung identifizierender Merkmale, spezielle Preisblatt-Varianten modelliert
    werden.

    Die jeweiligen Sätze von Merkmalen sind in der Grafik ergänzt worden und stellen jeweils eine Ausprägung für die
    verschiedenen Anwendungsfälle der Preisblätter dar.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Preisblatt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisblatt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Preisblatt.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.PREISBLATT, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    gueltigkeit: Zeitraum | None = None
    herausgeber: Marktteilnehmer | None = None
    preispositionen: Annotated[list[Preisposition] | None, Field(None, title="Preispositionen")]
    preisstatus: Preisstatus | None = None
    sparte: Sparte | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from enum import Enum


class Mengenoperator(str, Enum):
    """
    Angabe, wie eine Menge in Bezug auf einen Wert zu bilden ist.
    """

    KLEINER_ALS = "KLEINER_ALS"
    GROESSER_ALS = "GROESSER_ALS"
    GLEICH = "GLEICH"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .menge import Menge


class Vertragsteil(BaseModel):
    """
    Abbildung für einen Vertragsteil. Der Vertragsteil wird dazu verwendet,
    eine vertragliche Leistung in Bezug zu einer Lokation (Markt- oder Messlokation) festzulegen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Vertragsteil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertragsteil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Vertragsteil.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    lokation: Annotated[str | None, Field(None, title="Lokation")]
    maximale_abnahmemenge: Annotated[Menge | None, Field(None, alias="maximaleAbnahmemenge")]
    minimale_abnahmemenge: Annotated[Menge | None, Field(None, alias="minimaleAbnahmemenge")]
    vertraglich_fixierte_menge: Annotated[Menge | None, Field(None, alias="vertraglichFixierteMenge")]
    vertragsteilbeginn: Annotated[datetime | None, Field(None, title="Vertragsteilbeginn")]
    vertragsteilende: Annotated[datetime | None, Field(None, title="Vertragsteilende")]



"""
Contains the dataset for the usage loader.
It also contains the validation logic for the usage loader dataset.
"""
from bomf.model import Bo4eDataSet

from ibims.bo4e import Energiemenge, Messlokation, Zaehler


class TripicaUsageLoaderDataSet(Bo4eDataSet):
    """
    This is a bo4e data set that consists of
    * an Energiemenge
    * a Messlokation
    * a Zaehler.
    In the context of this package it contains all information to construct
    the tripica loader model.
    """

    energiemenge: Energiemenge
    messlokation: Messlokation
    zaehler: Zaehler



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.adresse import Adresse
from ..com.externe_referenz import ExterneReferenz
from ..enum.anrede import Anrede
from ..enum.bo_typ import BoTyp
from ..enum.geschaeftspartnerrolle import Geschaeftspartnerrolle
from ..enum.kontaktart import Kontaktart
from ..enum.marktrolle import Marktrolle
from ..enum.rollencodetyp import Rollencodetyp
from ..enum.sparte import Sparte


class Marktteilnehmer(BaseModel):
    """
    Objekt zur Aufnahme der Information zu einem Marktteilnehmer

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Marktteilnehmer.svg" type="image/svg+xml"></object>

    .. HINT::
        `Marktteilnehmer JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Marktteilnehmer.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    amtsgericht: Annotated[str | None, Field(None, title="Amtsgericht")]
    anrede: Anrede | None = None
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.MARKTTEILNEHMER, alias="boTyp")]
    e_mail_adresse: Annotated[str | None, Field(None, alias="eMailAdresse", title="Emailadresse")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    geschaeftspartnerrolle: Annotated[list[Geschaeftspartnerrolle] | None, Field(None, title="Geschaeftspartnerrolle")]
    gewerbekennzeichnung: Annotated[bool | None, Field(None, title="Gewerbekennzeichnung")]
    glaeubiger_id: Annotated[str | None, Field(None, alias="glaeubigerId", title="Glaeubigerid")]
    hrnummer: Annotated[str | None, Field(None, title="Hrnummer")]
    kontaktweg: Annotated[list[Kontaktart] | None, Field(None, title="Kontaktweg")]
    makoadresse: Annotated[str | None, Field(None, title="Makoadresse")]
    marktrolle: Marktrolle | None = None
    name1: Annotated[str | None, Field(None, title="Name1")]
    name2: Annotated[str | None, Field(None, title="Name2")]
    name3: Annotated[str | None, Field(None, title="Name3")]
    partneradresse: Adresse | None = None
    rollencodenummer: Annotated[str | None, Field(None, title="Rollencodenummer")]
    rollencodetyp: Rollencodetyp | None = None
    sparte: Sparte | None = None
    umsatzsteuer_id: Annotated[str | None, Field(None, alias="umsatzsteuerId", title="Umsatzsteuerid")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    website: Annotated[str | None, Field(None, title="Website")]



from enum import Enum


class HinweisThema(str, Enum):
    """
    Topics for the hints
    """

    SCHLECHTZAHLER = "SCHLECHTZAHLER"
    STROMLIEBHABER = "STROMLIEBHABER"
    STORNIERUNG = "STORNIERUNG"
    STAMMDATEN = "STAMMDATEN"
    LEGAL = "LEGAL"
    KUENDIGUNG = "KUENDIGUNG"
    BESCHWERDE = "BESCHWERDE"
    WIDERRUF = "WIDERRUF"
    KUNDENPORTAL = "KUNDENPORTAL"
    PREISANPASSUNG = "PREISANPASSUNG"
    ANMELDUNG = "ANMELDUNG"
    BONUS = "BONUS"
    LIEFERANTENWECHSEL = "LIEFERANTENWECHSEL"
    MAHNUNG = "MAHNUNG"
    RECHNUNG = "RECHNUNG"
    UMZUG = "UMZUG"
    VERTRAG = "VERTRAG"
    ZAHLUNG = "ZAHLUNG"
    ZAEHLERSTAND = "ZAEHLERSTAND"
    ABSCHLAG = "ABSCHLAG"
    CLEARINGDATEN = "CLEARINGDATEN"
    KLAERUNGMARKTPARTNER = "KLAERUNGMARKTPARTNER"
    MEHRVERBRAUCHTROCKNUNGSMASSNAHME = "MEHRVERBRAUCHTROCKNUNGSMASSNAHME"
    RECHNUNGSKORREKTUR = "RECHNUNGSKORREKTUR"
    REKLAMATION = "REKLAMATION"
    VERTRIEBSPARTNER = "VERTRIEBSPARTNER"
    VOLLMACHT = "VOLLMACHT"
    ENERGIEPREISMASSNAHMEN = "ENERGIEPREISMASSNAHMEN"
    ANDERE = "ANDERE"
    MARKETINGKAMPAGNE = "MARKETINGKAMPAGNE"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.auf_abschlagstyp import AufAbschlagstyp
from ..enum.auf_abschlagsziel import AufAbschlagsziel
from ..enum.waehrungseinheit import Waehrungseinheit
from .preisstaffel import Preisstaffel
from .zeitraum import Zeitraum


class AufAbschlag(BaseModel):
    """
    Modell für die preiserhöhenden (Aufschlag) bzw. preisvermindernden (Abschlag) Zusatzvereinbarungen,
    die individuell zu einem neuen oder bestehenden Liefervertrag abgeschlossen wurden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/AufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `AufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/AufAbschlag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    auf_abschlagstyp: Annotated[AufAbschlagstyp | None, Field(None, alias="aufAbschlagstyp")]
    auf_abschlagsziel: Annotated[AufAbschlagsziel | None, Field(None, alias="aufAbschlagsziel")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bezeichnung: Annotated[str | None, Field(None, title="Bezeichnung")]
    einheit: Waehrungseinheit | None = None
    gueltigkeitszeitraum: Zeitraum | None = None
    staffeln: Annotated[list[Preisstaffel] | None, Field(None, title="Staffeln")]
    website: Annotated[str | None, Field(None, title="Website")]



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from .sigmoidparameter import Sigmoidparameter


class Preisstaffel(BaseModel):
    """
    Gibt die Staffelgrenzen der jeweiligen Preise an

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preisstaffel.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisstaffel JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Preisstaffel.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    einheitspreis: Annotated[Decimal | None, Field(None, title="Einheitspreis")]
    sigmoidparameter: Sigmoidparameter | None = None
    staffelgrenze_bis: Annotated[Decimal | None, Field(None, alias="staffelgrenzeBis", title="Staffelgrenzebis")]
    staffelgrenze_von: Annotated[Decimal | None, Field(None, alias="staffelgrenzeVon", title="Staffelgrenzevon")]



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.lastprofil import Lastprofil
from ..enum.aggregationsverantwortung import Aggregationsverantwortung
from ..enum.bo_typ import BoTyp
from ..enum.profiltyp import Profiltyp
from ..enum.prognosegrundlage import Prognosegrundlage


class Bilanzierung(BaseModel):
    """
    Bilanzierung is a business object used for balancing. This object is no BO4E standard and a complete go
    implementation can be found at
    https://github.com/Hochfrequenz/go-bo4e/blob/3414a1eac741542628df796d6beb43eaa27b0b3e/bo/bilanzierung.go
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    bo_typ: Annotated[BoTyp | None, Field("BILANZIERUNG", alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bilanzierungsbeginn: Annotated[datetime, Field(title="Bilanzierungsbeginn")]
    bilanzierungsende: Annotated[datetime, Field(title="Bilanzierungsende")]
    bilanzkreis: Annotated[str | None, Field(None, title="Bilanzkreis")]
    aggregationsverantwortung: Aggregationsverantwortung | None = None
    lastprofile: Annotated[list[Lastprofil] | None, Field(None, title="Lastprofile")]
    prognosegrundlage: Prognosegrundlage | None = None
    details_prognosegrundlage: Annotated[Profiltyp | None, Field(None, alias="detailsPrognosegrundlage")]



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.bdew_artikelnummer import BDEWArtikelnummer
from ..enum.bemessungsgroesse import Bemessungsgroesse
from ..enum.kalkulationsmethode import Kalkulationsmethode
from ..enum.leistungstyp import Leistungstyp
from ..enum.mengeneinheit import Mengeneinheit
from ..enum.steuerkennzeichen import Steuerkennzeichen
from ..enum.tarifzeit import Tarifzeit
from ..enum.waehrungseinheit import Waehrungseinheit
from ..enum.zeiteinheit import Zeiteinheit
from .preisstaffel import Preisstaffel


class Preisposition(BaseModel):
    """
    Preis für eine definierte Lieferung oder Leistung innerhalb eines Preisblattes

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preisposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Preisposition.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bdew_artikelnummer: Annotated[BDEWArtikelnummer | None, Field(None, alias="bdewArtikelnummer")]
    berechnungsmethode: Kalkulationsmethode | None = None
    bezugsgroesse: Mengeneinheit | None = None
    freimenge_blindarbeit: Annotated[
        Decimal | None, Field(None, alias="freimengeBlindarbeit", title="Freimengeblindarbeit")
    ]
    freimenge_leistungsfaktor: Annotated[
        Decimal | None, Field(None, alias="freimengeLeistungsfaktor", title="Freimengeleistungsfaktor")
    ]
    gruppenartikel_id: Annotated[str | None, Field(None, alias="gruppenartikelId", title="Gruppenartikelid")]
    leistungsbezeichnung: Annotated[str | None, Field(None, title="Leistungsbezeichnung")]
    leistungstyp: Leistungstyp | None = None
    preiseinheit: Waehrungseinheit | None = None
    preisstaffeln: Annotated[list[Preisstaffel] | None, Field(None, title="Preisstaffeln")]
    tarifzeit: Tarifzeit | None = None
    zeitbasis: Zeiteinheit | None = None
    zonungsgroesse: Bemessungsgroesse | None = None
    steuersatz: Steuerkennzeichen



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class StandorteigenschaftenStrom(BaseModel):
    """
    Standorteigenschaften der Sparte Strom

    .. raw:: html

        <object data="../_static/images/bo4e/com/StandorteigenschaftenStrom.svg" type="image/svg+xml"></object>

    .. HINT::
        `StandorteigenschaftenStrom JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/StandorteigenschaftenStrom.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bilanzierungsgebiet_eic: Annotated[
        str | None, Field(None, alias="bilanzierungsgebietEic", title="Bilanzierungsgebieteic")
    ]
    regelzone: Annotated[str | None, Field(None, title="Regelzone")]
    regelzone_eic: Annotated[str | None, Field(None, alias="regelzoneEic", title="Regelzoneeic")]



from enum import Enum


class Tarifzeit(str, Enum):
    """
    Zur Kennzeichnung verschiedener Tarifzeiten, beispielsweise zur Bepreisung oder zur Verbrauchsermittlung.
    """

    TZ_STANDARD = "TZ_STANDARD"
    TZ_HT = "TZ_HT"
    TZ_NT = "TZ_NT"



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.messpreistyp import Messpreistyp
from ..enum.tarifkalkulationsmethode import Tarifkalkulationsmethode
from .preis import Preis
from .tarifpreis import Tarifpreis


class Tarifberechnungsparameter(BaseModel):
    """
    In dieser Komponente sind die Berechnungsparameter für die Ermittlung der Tarifkosten zusammengefasst.
    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifberechnungsparameter.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifberechnungsparameter JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Tarifberechnungsparameter.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    berechnungsmethode: Tarifkalkulationsmethode | None = None
    hoechstpreis_ht: Annotated[Preis | None, Field(None, alias="hoechstpreisHT")]
    hoechstpreis_nt: Annotated[Preis | None, Field(None, alias="hoechstpreisNT")]
    kw_inklusive: Annotated[Decimal | None, Field(None, alias="kwInklusive", title="Kwinklusive")]
    kw_weitere_mengen: Annotated[Decimal | None, Field(None, alias="kwWeitereMengen", title="Kwweiteremengen")]
    messpreis_beruecksichtigen: Annotated[
        bool | None, Field(None, alias="messpreisBeruecksichtigen", title="Messpreisberuecksichtigen")
    ]
    messpreis_in_gp_enthalten: Annotated[
        bool | None, Field(None, alias="messpreisInGpEnthalten", title="Messpreisingpenthalten")
    ]
    messpreistyp: Messpreistyp | None = None
    mindestpreis: Preis | None = None
    zusatzpreise: Annotated[list[Tarifpreis] | None, Field(None, title="Zusatzpreise")]



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.landescode import Landescode


class Adresse(BaseModel):
    """
    Contains an address that can be used for most purposes.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Adresse.svg" type="image/svg+xml"></object>

    .. HINT::
        `Adresse JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Adresse.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    adresszusatz: Annotated[str | None, Field(None, title="Adresszusatz")]
    co_ergaenzung: Annotated[str | None, Field(None, alias="coErgaenzung", title="Coergaenzung")]
    hausnummer: Annotated[str | None, Field(None, title="Hausnummer")]
    landescode: Landescode | None = Landescode.DE
    ort: Annotated[str | None, Field(None, title="Ort")]
    ortsteil: Annotated[str | None, Field(None, title="Ortsteil")]
    postfach: Annotated[str | None, Field(None, title="Postfach")]
    postleitzahl: Annotated[str | None, Field(None, title="Postleitzahl")]
    strasse: Annotated[str | None, Field(None, title="Strasse")]



from enum import Enum


class Kundentyp(str, Enum):
    """
    Auflistung der Typen von Endkunden. Daraus kann das Verbrauchsprofil abgeleitet werden.
    """

    GEWERBE = "GEWERBE"
    PRIVAT = "PRIVAT"
    LANDWIRT = "LANDWIRT"
    SONSTIGE = "SONSTIGE"
    HAUSHALT = "HAUSHALT"
    DIREKTHEIZUNG = "DIREKTHEIZUNG"
    GEMEINSCHAFT_MFH = "GEMEINSCHAFT_MFH"
    KIRCHE = "KIRCHE"
    KWK = "KWK"
    LADESAEULE = "LADESAEULE"
    BELEUCHTUNG_OEFFENTLICH = "BELEUCHTUNG_OEFFENTLICH"
    BELEUCHTUNG_STRASSE = "BELEUCHTUNG_STRASSE"
    SPEICHERHEIZUNG = "SPEICHERHEIZUNG"
    UNTERBR_EINRICHTUNG = "UNTERBR_EINRICHTUNG"
    WAERMEPUMPE = "WAERMEPUMPE"



from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.adresse import Adresse
from ..com.dienstleistung import Dienstleistung
from ..com.externe_referenz import ExterneReferenz
from ..com.geokoordinaten import Geokoordinaten
from ..com.hardware import Hardware
from ..com.katasteradresse import Katasteradresse
from ..enum.bo_typ import BoTyp
from ..enum.netzebene import Netzebene
from ..enum.sparte import Sparte
from .zaehler import Zaehler


class Messlokation(BaseModel):
    """
    Object containing information about a Messlokation

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Messlokation.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokation JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Messlokation.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.MESSLOKATION, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    geoadresse: Geokoordinaten | None = None
    geraete: Annotated[list[Hardware] | None, Field(None, title="Geraete")]
    grundzustaendiger_msb_codenr: Annotated[
        str | None, Field(None, alias="grundzustaendigerMsbCodenr", title="Grundzustaendigermsbcodenr")
    ]
    grundzustaendiger_msbim_codenr: Annotated[
        str | None, Field(None, alias="grundzustaendigerMsbimCodenr", title="Grundzustaendigermsbimcodenr")
    ]
    katasterinformation: Katasteradresse | None = None
    messadresse: Adresse | None = None
    messdienstleistung: Annotated[list[Dienstleistung] | None, Field(None, title="Messdienstleistung")]
    messgebietnr: Annotated[str | None, Field(None, title="Messgebietnr")]
    messlokations_id: Annotated[str | None, Field(None, alias="messlokationsId", title="Messlokationsid")]
    messlokationszaehler: Annotated[list[Zaehler] | None, Field(None, title="Messlokationszaehler")]
    netzebene_messung: Annotated[Netzebene | None, Field(None, alias="netzebeneMessung")]
    sparte: Sparte | None = None
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]



from enum import Enum


class BDEWArtikelnummer(str, Enum):
    """
    BDEW Artikelnummern
    """

    LEISTUNG = "LEISTUNG"
    LEISTUNG_PAUSCHAL = "LEISTUNG_PAUSCHAL"
    GRUNDPREIS = "GRUNDPREIS"
    REGELENERGIE_ARBEIT = "REGELENERGIE_ARBEIT"
    REGELENERGIE_LEISTUNG = "REGELENERGIE_LEISTUNG"
    NOTSTROMLIEFERUNG_ARBEIT = "NOTSTROMLIEFERUNG_ARBEIT"
    NOTSTROMLIEFERUNG_LEISTUNG = "NOTSTROMLIEFERUNG_LEISTUNG"
    RESERVENETZKAPAZITAET = "RESERVENETZKAPAZITAET"
    RESERVELEISTUNG = "RESERVELEISTUNG"
    ZUSAETZLICHE_ABLESUNG = "ZUSAETZLICHE_ABLESUNG"
    PRUEFGEBUEHREN_AUSSERPLANMAESSIG = "PRUEFGEBUEHREN_AUSSERPLANMAESSIG"
    WIRKARBEIT = "WIRKARBEIT"
    SINGULAER_GENUTZTE_BETRIEBSMITTEL = "SINGULAER_GENUTZTE_BETRIEBSMITTEL"
    ABGABE_KWKG = "ABGABE_KWKG"
    ABSCHLAG = "ABSCHLAG"
    KONZESSIONSABGABE = "KONZESSIONSABGABE"
    ENTGELT_FERNAUSLESUNG = "ENTGELT_FERNAUSLESUNG"
    UNTERMESSUNG = "UNTERMESSUNG"
    BLINDMEHRARBEIT = "BLINDMEHRARBEIT"
    ENTGELT_ABRECHNUNG = "ENTGELT_ABRECHNUNG"
    SPERRKOSTEN = "SPERRKOSTEN"
    ENTSPERRKOSTEN = "ENTSPERRKOSTEN"
    MAHNKOSTEN = "MAHNKOSTEN"
    MEHR_MINDERMENGEN = "MEHR_MINDERMENGEN"
    INKASSOKOSTEN = "INKASSOKOSTEN"
    BLINDMEHRLEISTUNG = "BLINDMEHRLEISTUNG"
    ENTGELT_MESSUNG_ABLESUNG = "ENTGELT_MESSUNG_ABLESUNG"
    ENTGELT_EINBAU_BETRIEB_WARTUNG_MESSTECHNIK = "ENTGELT_EINBAU_BETRIEB_WARTUNG_MESSTECHNIK"
    AUSGLEICHSENERGIE = "AUSGLEICHSENERGIE"
    ZAEHLEINRICHTUNG = "ZAEHLEINRICHTUNG"
    WANDLER_MENGENUMWERTER = "WANDLER_MENGENUMWERTER"
    KOMMUNIKATIONSEINRICHTUNG = "KOMMUNIKATIONSEINRICHTUNG"
    TECHNISCHE_STEUEREINRICHTUNG = "TECHNISCHE_STEUEREINRICHTUNG"
    PARAGRAF_19_STROM_NEV_UMLAGE = "PARAGRAF_19_STROM_NEV_UMLAGE"
    BEFESTIGUNGSEINRICHTUNG = "BEFESTIGUNGSEINRICHTUNG"
    OFFSHORE_HAFTUNGSUMLAGE = "OFFSHORE_HAFTUNGSUMLAGE"
    FIXE_ARBEITSENTGELTKOMPONENTE = "FIXE_ARBEITSENTGELTKOMPONENTE"
    FIXE_LEISTUNGSENTGELTKOMPONENTE = "FIXE_LEISTUNGSENTGELTKOMPONENTE"
    UMLAGE_ABSCHALTBARE_LASTEN = "UMLAGE_ABSCHALTBARE_LASTEN"
    MEHRMENGE = "MEHRMENGE"
    MINDERMENGE = "MINDERMENGE"
    ENERGIESTEUER = "ENERGIESTEUER"
    SMARTMETER_GATEWAY = "SMARTMETER_GATEWAY"
    STEUERBOX = "STEUERBOX"
    MSB_INKL_MESSUNG = "MSB_INKL_MESSUNG"
    AUSGLEICHSENERGIE_UNTERDECKUNG = "AUSGLEICHSENERGIE_UNTERDECKUNG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_1_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_1_MSBG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_2_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_2_MSBG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_3_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_3_MSBG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_4_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_4_MSBG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_5_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_2_5_MSBG"
    ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_3_MSBG = "ZUSATZDIENSTLEISTUNG_PARAGRAPH_35_3_MSBG"



from enum import Enum


class BoTyp(str, Enum):
    """
    Auflistung sämtlicher existierender Geschäftsobjekte.
    """

    ANGEBOT = "ANGEBOT"
    ANSPRECHPARTNER = "ANSPRECHPARTNER"
    AUSSCHREIUNG = "AUSSCHREIUNG"
    BUENDELVERTRAG = "BUENDELVERTRAG"
    ENERGIEMENGE = "ENERGIEMENGE"
    FREMDKOSTEN = "FREMDKOSTEN"
    GESCHAEFTSOBJEKT = "GESCHAEFTSOBJEKT"
    GESCHAEFTSPARTNER = "GESCHAEFTSPARTNER"
    KOSTEN = "KOSTEN"
    LASTGANG = "LASTGANG"
    LASTGANG_KOMPAKT = "LASTGANG_KOMPAKT"
    MARKTLOKATION = "MARKTLOKATION"
    MESSLOKATION = "MESSLOKATION"
    MARKTTEILNEHMER = "MARKTTEILNEHMER"
    NETZNUTZUNGSRECHNUNG = "NETZNUTZUNGSRECHNUNG"
    PREISBLATT = "PREISBLATT"
    PREISBLATTDIENSTLEISTUNG = "PREISBLATTDIENSTLEISTUNG"
    PREISBLATTHARDWARE = "PREISBLATTHARDWARE"
    PREISBLATTKONZESSIONSABGABE = "PREISBLATTKONZESSIONSABGABE"
    PREISBLATTMESSUNG = "PREISBLATTMESSUNG"
    PREISBLATTNETZNUTZUNG = "PREISBLATTNETZNUTZUNG"
    PREISBLATTUMLAGEN = "PREISBLATTUMLAGEN"
    RECHNUNG = "RECHNUNG"
    REGION = "REGION"
    REGIONALTARIF = "REGIONALTARIF"
    STANDORTEIGENSCHAFTEN = "STANDORTEIGENSCHAFTEN"
    TARIF = "TARIF"
    TARIFINFO = "TARIFINFO"
    TARIFKOSTEN = "TARIFKOSTEN"
    TARIFPREISBLATT = "TARIFPREISBLATT"
    VERTRAG = "VERTRAG"
    ZAEHLER = "ZAEHLER"
    ZEITREIHE = "ZEITREIHE"
    BILANZIERUNG = "Bilanzierung"
    DOKUMENT = "Dokument"
    FILE = "File"
    HINWEIS = "Hinweis"
    KAMPAGNE = "Kampagne"
    ZAEHLER_GAS = "ZaehlerGas"



"""
Contains the dataset for the network loader.
It also contains the validation logic for the network loader dataset.
"""
from typing import Optional

from ibims.bo4e import (
    Adresse,
    Bilanzierung,
    ConcessionFee,
    Geschaeftspartner,
    Marktlokation,
    Marktteilnehmer,
    Vertrag,
    Zaehler,
    ZaehlerGas,
    Zaehlpunkt,
)
from ibims.datasets import DataSetBaseModel


class TripicaNetworkLoaderDataSet(DataSetBaseModel):
    """
    This is a bo4e data set that consists of
    * two GeschaeftspartnerErweitert as customer
    * a Adresse
    * a Marktlokation
    * a Messlokation
    * two Marktteilnehmer
    * a Zaehler
    In the context of this package is may be used to create Tripica Network Data.
    """

    kunde: Geschaeftspartner
    """
    The following attributes need to be filled for this DataSet and is used for
    the customer that holds the contract:
    - name1 (Surname)
    - name2 (Firstname)
    - name3 (Title e.g. Dr.)
    - gewerbekennzeichnung
    - partneradresse
    """

    liefer_adresse: Adresse

    geschaeftspartner: Geschaeftspartner
    """
    This information is used for the customerMeterReadings part and
    is not a real business partner, so not all information of this object arte needed:
    - name1 (Surname)
    - name2 (Firstname)
    - partneradresse
    """
    # TODO: get a new table join with firstname and surname separated.
    # https://github.com/Hochfrequenz/powercloud2lynqtech/issues/818

    marktlokation: Marktlokation
    """
    The following attributes need to be filled for this DataSet:
    - marktlokations_id
    - sparte
    - bilanzierungsgebiet
    - unterbrechbar
    - gasqualitaet (if sparte is GAS)
    - netzbetreibercodenr
    - netzgebietsnr
    - kundengruppen
    - messtechnische_einordnung
    - ubertragungsnetzgebiet
    """

    netzbetreiber: Optional[Marktteilnehmer] = None
    """
    The following attributes need to be filled for this DataSet:
    - name1
    - rollencodetyp
    - rollencodenummer
    """

    zaehlpunkt: Zaehlpunkt

    messstellenbetreiber: Optional[Marktteilnehmer] = None
    """
    The following attributes need to be filled for this DataSet:
    - rollencodetyp
    - rollencodenummer
    """

    zaehler: Zaehler | ZaehlerGas
    """
    The following attributes need to be filled for this DataSet:
    - zaehlernummer
    - sparte
    - zaehlerauspraegung
    - zaehlertyp
    - zaehlergroeßeGas
    - tarifart
    - zaehlerauspraegung
    - zaehlwerke with each
    - zaehlwerk_id?
    - obis_kennzahl
    """

    vertrag: Vertrag
    """
    The following attributes need to be filled for this DataSet:
    - vertragsnummer (not mapped just for tracking)
    - vertragsbeginn
    - vertragsende
    """

    bilanzierung: Bilanzierung
    """
    The following attributes need to be filled for this DataSet:
    - bilanzkreis
    - bilanzierungsbeginn
    """

    concession_fees: list[ConcessionFee] | None

    slp_profilname: str | None

    tlp_profilname: str | None



from enum import Enum


class Rechnungstyp(str, Enum):
    """
    Abbildung verschiedener Rechnungstypen zur Kennzeichnung von Rechnungen
    """

    ENDKUNDENRECHNUNG = "ENDKUNDENRECHNUNG"
    NETZNUTZUNGSRECHNUNG = "NETZNUTZUNGSRECHNUNG"
    MEHRMINDERMENGENRECHNUNG = "MEHRMINDERMENGENRECHNUNG"
    MESSSTELLENBETRIEBSRECHNUNG = "MESSSTELLENBETRIEBSRECHNUNG"
    BESCHAFFUNGSRECHNUNG = "BESCHAFFUNGSRECHNUNG"
    AUSGLEICHSENERGIERECHNUNG = "AUSGLEICHSENERGIERECHNUNG"
    TURNUSRECHNUNG = "TURNUSRECHNUNG"
    ABSCHLAGSRECHNUNG = "ABSCHLAGSRECHNUNG"
    ABSCHLUSSRECHNUNG = "ABSCHLUSSRECHNUNG"
    ZWISCHENRECHNUNG = "ZWISCHENRECHNUNG"
    INTEGRIERTE_13_TE_RECHNUNG = "INTEGRIERTE_13TE_RECHNUNG"
    MONATSRECHNUNG = "MONATSRECHNUNG"
    ZUSAETZLICHE_13_TE_RECHNUNG = "ZUSAETZLICHE_13TE_RECHNUNG"



from enum import Enum


class Variant(str, Enum):
    """
    gridUsageBilling.variant (Netznutzungsabrechnungsvariante)
        Z14: WorkPriceBasicPrice (Arbeitspreis/Grundpreis)
        Z15: WorkPricePerformancePrice (Arbeitspreis/Leistungspreis)
    """

    Z14 = "Z14"
    Z15 = "Z15"



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..com.externe_referenz import ExterneReferenz
from ..com.unterschrift import Unterschrift
from ..com.vertragskonditionen import Vertragskonditionen
from ..com.vertragsteil import Vertragsteil
from ..enum.bo_typ import BoTyp
from ..enum.sparte import Sparte
from ..enum.vertragsart import Vertragsart
from ..enum.vertragsstatus import Vertragsstatus
from .geschaeftspartner import Geschaeftspartner


class Vertrag(BaseModel):
    """
    Modell für die Abbildung von Vertragsbeziehungen;
    Das Objekt dient dazu, alle Arten von Verträgen, die in der Energiewirtschaft Verwendung finden, abzubilden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Vertrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/bo/Vertrag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    beschreibung: Annotated[str | None, Field(None, title="Beschreibung")]
    bo_typ: Annotated[BoTyp | None, Field(BoTyp.VERTRAG, alias="boTyp")]
    externe_referenzen: Annotated[
        list[ExterneReferenz] | None, Field(None, alias="externeReferenzen", title="Externereferenzen")
    ]
    sparte: Sparte | None = None
    unterzeichnervp1: Annotated[list[Unterschrift] | None, Field(None, title="Unterzeichnervp1")]
    unterzeichnervp2: Annotated[list[Unterschrift] | None, Field(None, title="Unterzeichnervp2")]
    versionstruktur: Annotated[str | None, Field("2", title="Versionstruktur")]
    vertragsart: Vertragsart | None = None
    vertragsbeginn: Annotated[datetime | None, Field(None, title="Vertragsbeginn")]
    vertragsende: Annotated[datetime | None, Field(None, title="Vertragsende")]
    vertragskonditionen: Vertragskonditionen | None = None
    vertragsnummer: Annotated[str | None, Field(None, title="Vertragsnummer")]
    vertragspartner1: Geschaeftspartner | None = None
    vertragspartner2: Geschaeftspartner | None = None
    vertragsstatus: Vertragsstatus | None = None
    vertragsteile: Annotated[list[Vertragsteil] | None, Field(None, title="Vertragsteile")]



from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated

from ..enum.steuerkennzeichen import Steuerkennzeichen
from ..enum.waehrungscode import Waehrungscode


class Steuerbetrag(BaseModel):
    """
    Abbildung eines Steuerbetrages.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Steuerbetrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Steuerbetrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-python/main/json_schemas/com/Steuerbetrag.json>`_
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    basiswert: Annotated[Decimal | None, Field(None, title="Basiswert")]
    steuerkennzeichen: Steuerkennzeichen | None = None
    steuerwert: Annotated[Decimal | None, Field(None, title="Steuerwert")]
    waehrung: Waehrungscode | None = None
    steuerwert_vorausgezahlt: Annotated[
        Decimal | None, Field(None, alias="steuerwertVorausgezahlt", title="Steuerwertvorausgezahlt")
    ]



from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field
from typing_extensions import Annotated


class ConcessionFee(BaseModel):
    """
    The Concession Fee object was created during a migration project.
    It contains attributes needed for metering mapping.
    """

    model_config = ConfigDict(
        extra="allow",
        populate_by_name=True,
    )
    id: Annotated[str | None, Field(None, alias="_id", title=" Id")]
    market_location_id: Annotated[str, Field(alias="marketLocationId", title="Marketlocationid")]
    group: Annotated[str | None, Field(None, title="Group")]
    obis: Annotated[str, Field(title="Obis")]
    active_from: Annotated[datetime, Field(alias="activeFrom", title="Activefrom")]
    active_until: Annotated[datetime | None, Field(None, alias="activeUntil", title="Activeuntil")]
    ka: Annotated[str | None, Field(None, title="Ka")]
